//
//$Id: PerformanceMonster.java 17 2005-05-28 10:11:44Z olef $
//

package performancemonster;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.List;

import javax.swing.SwingUtilities;
import javax.swing.Timer;

import org.apache.log4j.Logger;

import performancemonster.analyzer.LogAnalyzer;
import performancemonster.loghandler.*;
import performancemonster.presentation.*;
import performancemonster.presentation.GraphElement;
import performancemonster.presentation.Model;

public class PerformanceMonster implements ActionListener {
    private static final Logger logger = Logger.getLogger(PerformanceMonster.class);
    private static MonsterGUI gui;
    
    public static void main(String[] args) {
        if (args.length != 1) {
            printUsageAndExit();
        }
        int portnumber;
        LogServer server;
        
        try {
            portnumber = Integer.parseInt(args[0]);
        } catch (NumberFormatException e) {
            System.out.println("Could not parse port number " + args[0]);
            printUsageAndExit();
            return;
        }
        
        try {
            server = new LogServer(portnumber);
        } catch (IOException e) {
            String errorString = "Error creating server at port " + portnumber;
            logger.error(errorString, e);
            throw new IllegalStateException(errorString, e);
        }

        final LogAnalyzer logAnalyzer = new LogAnalyzer();
        server.addLogListener(logAnalyzer);
        server.start();
            
        final Model myModel = new Model() {
            public void clear() {
                logAnalyzer.clear();
            }

            public List<GraphElement> getUpdatedList() {
                return logAnalyzer.analyzeLogs();
            }
        };

        gui = new MonsterGUI(myModel);
        
        //Set up the timer that will perform the animation.
        Timer timer = new Timer(100, new PerformanceMonster());
        timer.setInitialDelay(1000);
        timer.setCoalesce(false);
        timer.start(); //Start the animation.

        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                gui.createAndShowGUI();
            }
        });
    }
    
    private static void printUsageAndExit() {
        System.out.println("Usage: java performancemonster.Performancemonster <port>");
        System.out.println(" where <port>: Port number to listen on, e.g. 4446.");
        System.exit(0);
    }

    public void actionPerformed(ActionEvent e) {
        if (!gui.isFrozen()) {
            gui.update();
        }
    }
}
