//
// $Id: AnalyzedGraphElement.java 9 2005-05-25 20:06:23Z olef $
//

package performancemonster.analyzer;

import java.awt.Color;
import java.util.Date;

import performancemonster.presentation.GraphElement;


/**
 * A GraphElement implementation used by the loghandler package.
 */
public class AnalyzedGraphElement implements GraphElement {
    AnalyzedLoggingEvent beginningEvent;
    AnalyzedLoggingEvent endEvent;
    long end;
    
    /**
     * Creates a new AnalyzedGraphElement based on an AnalyzedLoggingEvent. The
     * "end" time will be set to System.currentTimeMillis().
     * 
     * @param event The wrapped AnalyzedLoggingEvent
     */
    public AnalyzedGraphElement(AnalyzedLoggingEvent event) {
        this.beginningEvent = event;
        this.end = System.currentTimeMillis();
    }

    /**
     * Gives a textual description of the element.
     */
    public String getDescription() {
        return beginningEvent.getName();
    }
    
    public String getLongDescription() {
        long duration = getEnd() - getBeginning();
        return beginningEvent.getName() + " (" + duration + " ms)\n"
            + beginningEvent.getMessage() + "\n"
            + endEvent.getMessage() + "\n\n"
            + beginningEvent.getStackTrace() + "\n"
            + "Start: " + new Date(getBeginning()) + "\n"
            + "End: " + new Date(getEnd());
    }

    /**
     * Gives the beginning timestamp for this element.
     */
    public long getBeginning() {
        return beginningEvent.getTimestamp();
    }

    /**
     * Gives the ending timestamp for this element.
     */
    public long getEnd() {
        return end;
    }
    
    /**
     * Sets the ending timestamp for this element.
     * 
     * @param end The ending timestamp, in "System.currentTimeMillis" format...
     */
    public void setEndEvent(AnalyzedLoggingEvent endEvent) {
        this.endEvent = endEvent;
        this.end = endEvent.getTimestamp();
    }

    /**
     * Currently, all colors are red...
     */
    public Color getColor() {
        return Color.RED;
    }
    
    /**
     * Gives the wrapped AnalyzedLoggingEvent.
     */
    public AnalyzedLoggingEvent getEvent() {
        return beginningEvent;
    }
    
    public String toString() {
        return getDescription();
    }
}