//
//$Id: AnalyzedLoggingEvent.java 9 2005-05-25 20:06:23Z olef $
//

package performancemonster.analyzer;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.StringTokenizer;

import org.apache.log4j.spi.LoggingEvent;

import performancemonster.loghandler.LoggingEventWrapper;

/**
 * Wrapper for a LoggingEvent. Gets a timestamp and a type (unknown, begin, or
 * end).
 */
class AnalyzedLoggingEvent {
    public static final int TYPE_UNKNOWN = 0;
    public static final int TYPE_BEGIN = 1;
    public static final int TYPE_END = 2;

    private long timestamp;
    private LoggingEventWrapper eventWrapper;
    private String name;
    private int type = TYPE_UNKNOWN;

    /**
     * Creates a new AnalyzedLoggingEvent based on the given LoggingEvent.
     * The type will be extracted from the message of the LoggingEvent, and
     * the timestamp will be taken from System.currentTimeMillis().
     */
    public AnalyzedLoggingEvent(LoggingEventWrapper eventWrapper) {
        this.eventWrapper = eventWrapper;
        this.timestamp = System.currentTimeMillis();
        String message = eventWrapper.getEvent().getMessage().toString();
        if (message.startsWith("BEGIN ")) {
            System.out.println("BEGIN: " + message);
            String messageBody = message.substring(6);
            StringTokenizer st = new StringTokenizer(messageBody, "(");
            name = st.nextToken();
            System.out.println("BEGIN: " + name);
            type = TYPE_BEGIN;
        } else if (message.startsWith("END ")) {
            System.out.println("END: " + message);
            String messageBody = message.substring(4);
            StringTokenizer st = new StringTokenizer(messageBody, "(");
            name = st.nextToken();
            System.out.println("END: " + name);
            type = TYPE_END;
        } else {
            type = TYPE_UNKNOWN;
        }
    }
    
    /**
     * Gives the timestamp when this object was created.
     */
    public long getTimestamp() {
        return timestamp;
    }
    
    /**
     * Gives the wrapped LoggingEvent.
     */
    public LoggingEvent getLoggingEvent() {
        return eventWrapper.getEvent();
    }
    
    public String getStackTrace() {
        StringWriter stringWriter = new StringWriter();
        PrintWriter printWriter = new PrintWriter(stringWriter);
        eventWrapper.getThrowable().printStackTrace(printWriter);
        return stringWriter.toString();
    }
    
    public String toString() {
        return "AnalyzedLoggingEvent[" + eventWrapper + "]";
    }
    
    /**
     * Gives the type (unknown, begin, end)
     */
    public int getType() {
        return type;
    }
    
    /**
     * Gives the name extracted from the wrapped LoggingEvent. The names are
     * used to couple beginning and ending log messages.
     */
    public String getName() {
        return name;
    }
    
    public String getMessage() {
        return eventWrapper.getEvent().getMessage().toString();
    }
}