//
//$Id: LogAnalyzer.java 15 2005-05-27 12:56:06Z olef $
//

package performancemonster.analyzer;

import java.util.*;

import performancemonster.loghandler.LogListener;
import performancemonster.loghandler.LoggingEventWrapper;
import performancemonster.presentation.GraphElement;

public class LogAnalyzer implements LogListener {
    private List<AnalyzedLoggingEvent> events = new ArrayList<AnalyzedLoggingEvent>();

    /**
     * Tells this objects that a new LoggingEvent has been received.
     */
    public void receiveLogEvent(LoggingEventWrapper eventWrapper) {
        AnalyzedLoggingEvent ale = new AnalyzedLoggingEvent(eventWrapper);
        if (ale.getType() != AnalyzedLoggingEvent.TYPE_UNKNOWN) {
            events.add(ale);
        } else {
            System.out.println("Ignoring " + eventWrapper);
        }
    }
    
    /**
     * Clears the underlying list of LoggingEvents.
     */
    public void clear() {
        events = new ArrayList<AnalyzedLoggingEvent>();
    }
    
    /**
     * Runs through all the gathered LoggingEvents, pairs up beginning and
     * ending LoggingEvents, and produces a list of GraphElement objects.
     */
    public List<GraphElement> analyzeLogs() {
        List<GraphElement> analyzedElements = new ArrayList<GraphElement>();
        List<AnalyzedGraphElement> elementStack = new ArrayList<AnalyzedGraphElement>();
        
        for (AnalyzedLoggingEvent event : events) {
            if (event.getType() == AnalyzedLoggingEvent.TYPE_BEGIN) {
                System.out.println("Begin " + event.getName());
                AnalyzedGraphElement element = new AnalyzedGraphElement(event);
                analyzedElements.add(element);
                elementStack.add(element);
            } else if (event.getType() == AnalyzedLoggingEvent.TYPE_END) {
                System.out.println("End " + event.getName());
                AnalyzedGraphElement element = findMatchingEvent(event, elementStack);
                if (element != null) {
                    element.setEndEvent(event);
                } else {
                    System.out.println("No match for " + event);
                }
            } else {
                System.out.println("Ignoring " + event);
            }
        }
        
        return analyzedElements;
    }
    
    /**
     * Finds the AnalyzedGraphElement from the given list, which mathes the
     * name from "event" and is a beginning event. Removes the event from the
     * given list. Searches backwards, in order to simulate a stack.
     */
    private AnalyzedGraphElement findMatchingEvent(AnalyzedLoggingEvent event, List<AnalyzedGraphElement> graphElements) {
        for (int i = graphElements.size() - 1; i >= 0; i--) {
            AnalyzedGraphElement element = graphElements.get(i);
            if (element.getEvent().getName().equals(event.getName())) {
                graphElements.remove(i);
                return element;
            }
        }
        return null;
    }
}
