//
//$Id: MonsterGUI.java 9 2005-05-25 20:06:23Z olef $
//

package performancemonster.presentation;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.*;

public class MonsterGUI {
    private JToggleButton zoomMode;
    private JToggleButton updateMode;
    private GraphArea graph;
    private JLabel info;
    private final Model model;
    
    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                MonsterGUI test = new MonsterGUI(null);
                test.createAndShowGUI();
            }
        });
    }
    
    public MonsterGUI(Model model) {
        this.model = model;
    }
    
    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    public void createAndShowGUI() {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (UnsupportedLookAndFeelException e) {
            e.printStackTrace();
        }

        //Create and set up the window.
        JFrame frame = new JFrame("Performance Monster");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        Action zoomOutAction = new AbstractAction("Zoom out") {
            public void actionPerformed(ActionEvent evt) {
                long oldBeginning = graph.getBeginning();
                long oldEnd = graph.getEnd();
                long diff = oldEnd - oldBeginning;
                graph.setBeginning(oldBeginning - diff / 2);
                graph.setEnd(oldEnd + diff / 2);
                graph.repaint();
            }
        };
        Action zoomModeAction = new AbstractAction("Zoom mode") {
            public void actionPerformed(ActionEvent evt) {
                graph.setZoomMode(zoomMode.isSelected());
            }
        };
        Action alignAction = new AbstractAction("Align") {
            public void actionPerformed(ActionEvent evt) {
                graph.align();
            }
        };
        Action updateAction = new AbstractAction("Update") {
            public void actionPerformed(ActionEvent evt) {
                //updateMode = updateMode.isSelected();
            }
        };
        Action clearAction = new AbstractAction("Clear") {
            public void actionPerformed(ActionEvent evt) {
                List<GraphElement> emptyList = new ArrayList<GraphElement>();
                model.clear();
                graph.setGraphElements(emptyList);
                graph.repaint();
            }
        };
        
        JButton zoomOut = new JButton(zoomOutAction);
        zoomMode = new JToggleButton(zoomModeAction);
        JButton reset = new JButton(alignAction);
        updateMode = new JToggleButton(updateAction);
        JButton clear = new JButton(clearAction);
        info = new JLabel("");
        graph = new GraphArea();
        
        graph.setBeginning(0L);
        graph.setEnd(300L);
        graph.addDescriptionListener(new DescriptionListener() {
            public void showDescription(GraphElement element) {
                System.out.println(element.getLongDescription());
            }
        });
        graph.addHoverListener(new HoverListener() {
            public void mouseHovered(long position, GraphElement element) {
                info.setText("Position " + (position - graph.getBeginning()) + "ms: " + element);
            }
        });
        
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        
        JPanel controls = new JPanel();
        controls.setLayout(new BoxLayout(controls, BoxLayout.LINE_AXIS));
        controls.add(zoomOut);
        controls.add(zoomMode);
        controls.add(reset);
        controls.add(updateMode);
        controls.add(clear);
        controls.add(info);

        panel.add(controls, BorderLayout.SOUTH);
        panel.add(graph, BorderLayout.CENTER);
        frame.add(panel);
        
        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }
    
    public void update() {
        List<GraphElement> updatedList = model.getUpdatedList();
        graph.setGraphElements(updatedList);
        graph.setEnd(System.currentTimeMillis());
        graph.align();
        graph.setEnd(System.currentTimeMillis());
    }
    
    public boolean isFrozen() {
        return !updateMode.isSelected();
    }
}
